library jwt_auth.example.models;

import 'dart:async';
import 'dart:convert';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_common/jaguar_common.dart';
import 'package:jaguar_auth/jaguar_auth.dart';

/// Model for Book
class Book {
  /// Id of the book
  String id;

  /// Name of the book
  String name;

  /// Authors of the book
  String author;

  Book.make(this.id, this.name, this.author);

  Book.FromQueryParam(QueryParams params) {
    fromMap({'name': params.name, 'authors': params.authors});
  }

  /// Converts to Map
  Map toMap() => {
        'id': id,
        'name': name,
        'author': author,
      };

  // Converts to JSON
  String toJson() {
    return JSON.encode(toMap());
  }

  /// Builds from JSON
  void fromJson(String json) {
    dynamic map = JSON.decode(json);

    if (map is Map) {
      fromMap(map);
    }
  }

  /// Builds from Map
  void fromMap(Map map) {
    if (map['id'] is String) {
      id = map['id'];
    }

    if (map['name'] is String) {
      name = map['name'];
    }

    if (map['author'] is String) {
      author = map['author'];
    }
  }
}

class User implements AuthorizationUser {
  final String id;

  final String username;

  final String password;

  const User(this.id, this.username, this.password);

  String get loginId => username;

  String get loginPassword => password;

  String get authorizationId => id;
}

/// Model manager to authenticate against a static list of user models
class WhiteListPasswordChecker implements AuthModelManager {
  /// User models to white list
  final Map<String, User> models;

  /// Password hasher
  final Hasher hasher;

  const WhiteListPasswordChecker(Map<String, User> models, {Hasher hasher})
      : models = models ?? const {},
        hasher = hasher ?? const NoHasher();

  Future<User> authenticate(String username, String password) async {
    User model = await fetchModelByAuthenticationId(username);

    if (model == null) {
      return null;
    }

    if (!hasher.verify(password, model.loginPassword)) {
      return null;
    }

    return model;
  }

  Future<User> fetchModelByAuthenticationId(String authName) async =>
      models.values
          .firstWhere((model) => model.loginId == authName, orElse: () => null);

  Future<User> fetchModelByAuthorizationId(String sessionId) async {
    if (!models.containsKey(sessionId)) {
      return null;
    }

    return models[sessionId];
  }
}
